/*
 * Run a script accessing a filesystem.
 *
 * Copyright (C) 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <string.h>

#include "input.h"
#include "ops.h"



/* Line buffer size. */

static int BUFSIZE = 4096;

/* Maximum number of arguments in scripts. */

static const int MAX_ARGS = 32;



/* Read operations from a script file. */

int run_script(int argc, char *argv[])
{
    FILE *fp;
    char buffer[BUFSIZE];
    enum op_results op_result;
    int num_args;
    int exitcode;
    char *args[MAX_ARGS];
    int i, use_stdin;

    for (i = 0; i < argc; i++)
    {
        /* Test for standard input as a special filename. */

        use_stdin = !strcmp(argv[i], "-");

        if (use_stdin)
            fp = stdin;
        else
            fp = fopen(argv[i], "r");

        while (1)
        {
            /* Show a prompt if employing standard input. */

            if (use_stdin)
                printf("> ");

            /* Read and parse the input line, running any specified command. */

            if (read_line(fp, buffer, BUFSIZE) == NULL)
                break;

            parse_line(buffer, buffer + strlen(buffer), &num_args, args, MAX_ARGS);

            /* Permit commands with and without arguments. */

            if (num_args >= 1)
            {
                op_result = run_operation(args[0], num_args - 1, &args[1], &exitcode);

                if (handle_op_result(args[0], op_result) && !use_stdin)
                    return 1;
            }
        }

        /* Close normal files. */

        if (!use_stdin)
            fclose(fp);
    }

    printf("End of script.\n");
    return 0;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
